#include <QFileDialog>
#include <QMessageBox>
#include <QDebug>
#include "flash_launcher.h"
#include "ui_flash_launcher.h"
#include "bin_file.h"
#include "datastream.h"
#include "settings.h"

flash_launcher::flash_launcher(datastream_control *control_in, datalog *log_in, QWidget *parent) :
  QWidget(parent),
  ui(new Ui::flash_launcher) {
  control = control_in;
  log = log_in;
  ui->setupUi(this);
  ui->patch_master->setChecked(config.get_flash_installpatch());
  ui->flash_skip_unused_regions->setChecked(config.get_flash_skip());
}

flash_launcher::~flash_launcher() {
  delete ui;
}

void flash_launcher::on_write_tside_toggled(bool checked) {
  control->flash_tside.set(checked);
}

void flash_launcher::on_write_eside_toggled(bool checked) {
  control->flash_eside.set(checked);
}

void flash_launcher::on_flash_skip_unused_regions_toggled(bool checked) {
  control->skip_unused_regions.set(checked);
  config.set_flash_skip(checked);
}

void flash_launcher::on_patch_master_toggled(bool checked) {
  control->enable_patches.set(checked);
  config.set_flash_installpatch(checked);
}

void flash_launcher::on_chk_dump_ram_toggled(bool checked) {
   control->dump_ram.set(checked);
}

void flash_launcher::compare_write_bins() {
  if(control->prev_bin.is_null() == true) {
    ui->write_eside->setChecked(true);
    ui->write_tside->setChecked(true);
    ui->write_eside->setEnabled(true);
    ui->write_tside->setEnabled(true);
  } else {
    ui->write_eside->setChecked(! control->write_bin.compare_eside(control->prev_bin));
    ui->write_tside->setChecked(! control->write_bin.compare_tside(control->prev_bin));
    ui->write_eside->setEnabled(false);
    ui->write_tside->setEnabled(false);
  }
}

void flash_launcher::on_patch_about_btn_clicked() {
   emit info_dialog(ui->patch_about_btn->toolTip());
}

void flash_launcher::on_bin_write_select_clicked() {
  QString filename = QFileDialog::getOpenFileName(
        this,"Select bin file to flash",
        config.get_savepath(),"BIN File (*.bin)");
  if(filename.isEmpty() == true || filename.isNull() == true) return;
  if(filename == nullptr || control->write_bin.load_file(filename) == false) {
    emit info_dialog("Can't load file, or bad bin!\n(Not an $EE bin?  Corrupt?)");
    ui->bin_write_filename->clear();
    ui->btn_writeflash->setEnabled(false);
    control->write_bin.zero_bin();
  } else {
    ui->bin_write_filename->setPlainText(filename);
    ui->btn_writeflash->setEnabled(true);
    compare_write_bins();
  }
}

void flash_launcher::on_btn_prev_write_select_clicked() {
  QString filename = QFileDialog::getOpenFileName(
        this,"Select bin file to COMPARE",
        config.get_savepath(),"BIN File (*.bin)");
  if(filename.isEmpty() == true || filename.isNull() == true) return;
  if(filename == nullptr || control->prev_bin.load_file(filename) == false) {
    emit info_dialog("Can't load file, or bad bin!\n(Not an $EE bin?  Corrupt?)");
    ui->bin_compare_filename->clear();
    ui->write_tside->setChecked(true);
    ui->write_eside->setChecked(true);
    ui->write_tside->setEnabled(true);
    ui->write_eside->setEnabled(true);
    control->prev_bin.zero_bin();
    config.set_flash_comparebin(nullptr);
  } else {
    ui->bin_compare_filename->setPlainText(filename);
    compare_write_bins();
    config.set_flash_comparebin(filename); // remember
  }
}

void flash_launcher::load_saved_compare_bin() {
  if(config.get_flash_comparebin() != nullptr) {
    ui->bin_compare_filename->setPlainText(config.get_flash_comparebin());
    if(control->prev_bin.load_file(ui->bin_compare_filename->toPlainText()) == true) {
      compare_write_bins();
    } else { // file can't be loaded, forget the bin.
      config.set_flash_comparebin(nullptr);
      ui->bin_compare_filename->clear();
    }
  } else {
    ui->bin_compare_filename->clear();
  }
}

void flash_launcher::on_btn_prev_clr_clicked() {
  ui->bin_compare_filename->clear();
  ui->write_tside->setChecked(true);
  ui->write_eside->setChecked(true);
  ui->write_tside->setEnabled(true);
  ui->write_eside->setEnabled(true);
  control->prev_bin.zero_bin();
  config.set_flash_comparebin(nullptr);
}

void flash_launcher::on_btn_readflash_clicked() {
  if(control->is_connected() == false) {
    emit info_dialog("Please establish a connection before programming.");
    return;
  }
  control->start_flash_read.request();
}

void flash_launcher::on_btn_writeflash_clicked() {
  if(control->is_connected() == false) {
    emit info_dialog("Please establish a connection before programming.");
    return;
  }
  // display warning text
  int ret = QMessageBox::warning(this, "EEHack Flash Pre-Warning",
                                 "BE WARNED!!!\n\
If anything happens during flashing, if your battery dies, your computer crashes, \
or you knock the cable out, you will BRICK YOUR ECM!\n\
You should run the stability test if you haven't already.\n\
If you sure you want to continue?",
                                 QMessageBox::Yes | QMessageBox::No,
                                 QMessageBox::Yes);
  if(ret == QMessageBox::No) return;
  control->start_flash_write.request();
}

void flash_launcher::on_interface_loaded() {
  control->flash_tside.set(ui->write_tside->isChecked());
  control->flash_eside.set(ui->write_eside->isChecked());
  control->skip_unused_regions.set(ui->flash_skip_unused_regions->isChecked());
  control->enable_patches.set(ui->patch_master->isChecked());
  control->dump_ram.set(ui->chk_dump_ram->isChecked());
  load_saved_compare_bin();
}

void flash_launcher::save_read_bin() {
  save_bin_file(&(control->read_bin));
}

void flash_launcher::remember_previous_bin_write() {
  ui->bin_compare_filename->setPlainText(ui->bin_write_filename->toPlainText());
  if(control->prev_bin.load_file(ui->bin_write_filename->toPlainText()) == true) {
    config.set_flash_comparebin(ui->bin_write_filename->toPlainText());
  } else {
    ui->bin_compare_filename->clear();
    config.set_flash_comparebin(nullptr);
  }
  compare_write_bins();
}

void flash_launcher::remember_previous_bin_read(QString filename) {
  ui->bin_compare_filename->setPlainText(filename);
  if(control->prev_bin.load_file(filename) == true) {
    config.set_flash_comparebin(filename);
  } else {
    ui->bin_compare_filename->clear();
    config.set_flash_comparebin(nullptr);
  }
  compare_write_bins();
}

void flash_launcher::save_bin_file(bin_file *bin) {
  if(bin == nullptr) {
    emit error_dialog("No data!");
    return;
  }
  QString filename = get_save_filename("bin","Tunerpro/Tunercat Bin");
  if(filename.isEmpty() == true || filename.isNull() == true) return;
  if(bin->save_file(filename) == false) {
    emit error_dialog("Could not open file for writing.");
  } else {
    emit info_dialog("Bin file saved.");
    remember_previous_bin_read(filename);
  }
}

QString flash_launcher::get_save_filename(QString extension, QString type_name) {
  QFileDialog dialog;
  dialog.setFileMode(QFileDialog::AnyFile);
  dialog.setDefaultSuffix(extension);
  dialog.setNameFilter(type_name + " (*." + extension + ")");
  dialog.setAcceptMode(QFileDialog::AcceptSave);
  dialog.setDirectory(config.get_savepath());
  QStringList file_names;
  if(dialog.exec()) {
    file_names = dialog.selectedFiles();
    return file_names.takeFirst();
  } else {
    return QString();
  }
}

void flash_launcher::on_btn_readflash_2_clicked() {
  if(control->is_connected() == false) {
    emit info_dialog("Please establish a connection before programming.");
    return;
  }
  control->start_flash_stability_test.request();
}
