#include "flashprogress.h"
#include "ui_flashprogress.h"
#include <QCloseEvent>

flashprogress::flashprogress(QWidget *parent) :
  QWidget(parent),
  ui(new Ui::flashprogress)
{
  ui->setupUi(this);
  cancelled = false;
  n_errors = 0;
}

flashprogress::~flashprogress() {
  delete ui;
}

void flashprogress::set_flash_speed(float bps) {
  if(bps == 0) {
    flash_speed_avg = 0;
    flash_speed_count = 0;
    ui->flash_speed_display->setText("---");
  } else {
    flash_speed_avg += bps;
    flash_speed_count++;
    if(flash_speed_count > 5) {
      if(flash_speed_avg > 100000) {
        ui->flash_speed_display->setText("Fast!");
      } else {
        ui->flash_speed_display->setText(QString::number(
                                           flash_speed_avg / flash_speed_count,
                                           'f',0) + "bps");
      }
      flash_speed_avg = 0;
      flash_speed_count = 0;
    }
  }
}

void flashprogress::closeEvent(QCloseEvent *x) {
  x->ignore();
}

void flashprogress::on_cancel_btn_clicked() {
  emit(click_cancel_button());
  cancelled = true;
}

void flashprogress::append_log(QString msg) {
  ui->flashlog->append(msg);
}

void flashprogress::append_error(QString msg) {
  append_log(QString("<font color=red>") + msg + QString("</font>"));
}

void flashprogress::append_divider(QString msg) {
  append_log(QString("----------------------------"));
  append_log(QString("<font color=blue>") + msg.toUpper() + QString("</font>"));
}

void flashprogress::clear_log() {
  ui->flashlog->clear();
}

void flashprogress::cancel_enable(bool value) {
  ui->cancel_btn->setEnabled(value);
}

bool flashprogress::is_cancelled() {
  return cancelled;
}

int flashprogress::set_eside_slider(int value) {
  if(value < 0x2000) value = 0x2000;
  ui->eside_progress->setValue(value);
  ui->display_eside->setText(QString::number(value,16).toUpper());
  return value;
}

int flashprogress::set_tside_slider(int value) {
  if(value < 0x2000) value = 0x2000;
  ui->tside_progress->setValue(value);
  ui->display_tside->setText(QString::number(value,16).toUpper());
  return value;
}

//---------------

void flashprogress::recieve_tside_progress(int value) {
  set_tside_slider(value);
}

void flashprogress::recieve_eside_progress(int value) {
  set_eside_slider(value);
}

void flashprogress::recieve_flash_speed(float value) {
  set_flash_speed(value);
}

void flashprogress::reset_interface() {
  ui->flashlog->clear();
  ui->flash_error_count->setText(QString::number(0));
  cancel_enable(false);
  cancelled = false;
  n_errors = 0;
  set_eside_slider(0);
  set_tside_slider(0);
}

void flashprogress::recieve_message(QString msg) {
  append_log(msg);
}

void flashprogress::recieve_divider(QString msg) {
  append_divider(msg);
}

void flashprogress::recieve_error_message(QString msg) {
  append_error(msg);
}

void flashprogress::recieve_cancel_enable(bool enable) {
  cancel_enable(enable);
}

void flashprogress::error_occurred() {
  n_errors++;
  ui->flash_error_count->setText(QString::number(n_errors));
}
